package robots;

import java.util.*;

/**
 * Trieda obsahujuca metodu main, realizujuca nacitanie vstupu a vytvorenie potrebnych objektov.
 */
public class Run {

    private static List<List<Boolean>> scanTorus(Scanner scanner) {
        int height = scanner.nextInt();
        int width = scanner.nextInt();
        scanner.nextLine();
        List<List<Boolean>> result = new ArrayList<>();
        for (int i = 0; i <= height - 1; i++) {
            String line = scanner.nextLine();
            result.add(new ArrayList<>());
            for (int j = 0; j <= width - 1; j++) {
                switch (line.charAt(j)) {
                    case 'T':
                        result.get(i).add(true);
                        break;
                    case 'F':
                        result.get(i).add(false);
                        break;
                }
            }
        }
        return result;
    }

    private static Robot createRobot(Scanner scanner) {
        Robot robot = null;
        String type = scanner.next();
        String reversed = scanner.next();
        int transitionCount;
        switch (type) {
            case "FiniteStateSetRobotProgram-Integer":
                transitionCount = scanner.nextInt();
                HashMap<StateSetRobotConfiguration<Integer>, StateSetRobotStep<Integer>> integerTransitions = new HashMap<>();
                for (int i = 1; i <= transitionCount; i++) {
                    int currentState = scanner.nextInt();
                    boolean currentCellContents = scanner.nextBoolean();
                    int newState = scanner.nextInt();
                    boolean newCellContents = scanner.nextBoolean();
                    Direction direction = Direction.valueOf(scanner.next());
                    integerTransitions.put(new StateSetRobotConfiguration<>(currentState, currentCellContents),
                            new StateSetRobotStep<>(newState, new Move(newCellContents, direction)));
                }
                int integerInitialState = scanner.nextInt();
                if (reversed.equals("normal")) {
                    robot = new StateSetRobot<>(new FiniteStateSetRobotProgram<>(integerTransitions, integerInitialState));
                } else if (reversed.equals("reversed")) {
                    robot = new ReversedStateSetRobot<>(new FiniteStateSetRobotProgram<>(integerTransitions, integerInitialState));
                }
                break;
            case "FiniteStateSetRobotProgram-String":
                transitionCount = scanner.nextInt();
                HashMap<StateSetRobotConfiguration<String>, StateSetRobotStep<String>> stringTransitions = new HashMap<>();
                for (int i = 1; i <= transitionCount; i++) {
                    String currentState = scanner.next();
                    boolean currentCellContents = scanner.nextBoolean();
                    String newState = scanner.next();
                    boolean newCellContents = scanner.nextBoolean();
                    Direction direction = Direction.valueOf(scanner.next());
                    stringTransitions.put(new StateSetRobotConfiguration<>(currentState, currentCellContents),
                            new StateSetRobotStep<>(newState, new Move(newCellContents, direction)));
                }
                String stringInitialState = scanner.next();
                if (reversed.equals("normal")) {
                    robot = new StateSetRobot<>(new FiniteStateSetRobotProgram<>(stringTransitions, stringInitialState));
                } else if (reversed.equals("reversed")) {
                    robot = new ReversedStateSetRobot<>(new FiniteStateSetRobotProgram<>(stringTransitions, stringInitialState));
                }
                break;
            case "MarkInitialCellProgram":
                if (reversed.equals("normal")) {
                    robot = new StateSetRobot<>(new MarkInitialCellProgram());
                }
                break;
            case "CellInvertingProgram":
                if (reversed.equals("normal")) {
                    robot = new StateSetRobot<>(new CellInvertingProgram(scanner.nextInt()));
                } else if (reversed.equals("reversed")) {
                    robot = new ReversedStateSetRobot<>(new CellInvertingProgram(scanner.nextInt()));
                }
                break;
        }
        return robot;
    }

    public static void main(String[] args) {
        Scanner scanner = new Scanner(System.in);
        List<List<Boolean>> torus = scanTorus(scanner);
        Robot robot = createRobot(scanner);
        Simulation simulation = new Simulation(torus, robot, scanner.nextInt(), scanner.nextInt());
        simulation.start(System.out, scanner.nextInt(), scanner.next());
    }

}
