package robots;

/**
 * Hotova genericka trieda reprezentujuca robota so stavmi typu T.
 */
public class StateSetRobot<T> implements Robot {
    private StateSetRobotProgram<T> program;
    private T currentState;

    /**
     * Argumentom konstruktora je program robota.
     */
    public StateSetRobot(StateSetRobotProgram<T> program) {
        this.program = program;
        currentState = program.getInitialState();
    }

    /**
     * Metoda implementujuca metodu hasNextMove z rozhrania Robot.
     */
    @Override
    public boolean hasNextMove(Boolean currentCellContents) {
        return program.nextStep(currentState, currentCellContents) != null;
    }

    /**
     * Metoda implementujuca metodu nextMove z rozhrania Robot. Nasledujuci pohyb robota sa vzdy urci na zaklade jeho
     * stavu a obsahu citaneho policka pomocou metody program.nextStep. Sucasne tato metoda realizuje aj zmenu stavu
     * robota na novy stav, ktory je taktiez dany vystupom metody program.nextStep.
     */
    @Override
    public Move nextMove(Boolean currentCellContents) {
        if (hasNextMove(currentCellContents)) {
            StateSetRobotStep<T> nextStep = program.nextStep(currentState, currentCellContents);
            currentState = nextStep.getNewState();
            return nextStep.getMove();
        } else {
            throw new NoNextMoveException();
        }
    }

}
